
// $Id: converter_core.js,v 1.55 2009/11/18 23:32:56 bogdan Exp $

/*

TO DO:
 - metric weights => imperial weights
 - surfaces
 - volumes
 - update example page with speeds plus everything above
 - example page: adaptive results -- in2m, m2in, kmph2miph, miph2kmph, lb2kg
*/

if (MCE!=undefined && MCE.applicationName=='Converter') {

MCE.core={
	/*
	* This function is the conversion brain -- it takes in a string, checks for
	* values to convert, converts them, executes the adjusters, and if all goes
	* well returns a converter result object, or false otherwise.
	*/
	/*
	* TODO: investigate why we need both parameters
	*/
	doConversion:function(txt,sw)
	{
	  if (sw) {
	    var temp = sw;
	  } else {
	    var temp = this.getSelectedWord(txt);
	    if (!temp) {
	      return false;
	    }
	  }
	  var pos=MCE.util.inArray(temp.unit,MCE.conversions[0]);
	  if (pos==-1) {
	    return false;
	  }

	  // Ok, it's legitimate, but let's check if we need to convert this at all
	  var target=MCE.conversions[2][pos];
	  if (MCE.prefs.getPref('pref_one_way') && target &&
	    (
		  (target=='metric' && !MCE.prefs.getPref('pref_metric')) ||
		  (target=='Imperial' && MCE.prefs.getPref('pref_metric')) ||
		  (target=='Celsius' && !MCE.prefs.getPref('pref_celsius')) ||
		  (target=='Fahrenheit' && MCE.prefs.getPref('pref_celsius'))
		)
	  ) {
	    return {inhibit:true};
	  }

	  if (temp.unit!='internal_time') {
	    temp.value=MCE.util.determine_separators(temp.value);
	  }
	  if (temp.value===false || temp.value===undefined) {
	    return false;
	  }

	  var converter=MCE.conversions[1][pos];
	  var cr=MCE.converters[converter](temp); // conversion result

	  if (temp.multiplier>1) {
	    cr.inValue*=temp.multiplier;
	    cr.outValue*=temp.multiplier;
	  }
	  // This is rather academic, since we're only using it once below, but hey!
	  // It's a hobby project!
	  if (cr.outUnit=="m") {
	    cr=MCE.adjusters.metricDistance(cr);
	  }
	  if (cr.outUnit=="kg") {
	    cr=MCE.adjusters.metricWeight(cr);
	  }
	  if (cr.outUnit=="in") {
	    cr=MCE.adjusters.imperialDistance(cr);
	  }
	  if ((cr.outUnit=='gal') || (cr.outUnit=='fl oz')) {
	    cr=MCE.adjusters.imperialVolume(cr);
	  }
	  if ((cr.outUnit=='ml') || (cr.outUnit=='L')) {
	    cr=MCE.adjusters.metricVolume(cr);
	  }
	  if (!cr.not_numerical) {
	    cr.inValue=MCE.util.smartRound(cr.inValue);
	    cr.outValue=MCE.util.smartRound(cr.outValue);
	  }
	  if (!cr.outCustom) {
	    cr.outCustom=cr.outValue+" "+cr.outUnit;
	  }
	  return cr;
	},

	/*
	* This function registers the converters defined at the beginning of this
	* file. What it basically does is split that string into items and builds
	* two arrays, one of which contains the units and the other the converters.
	* It then builds an array with two elements which are the arrays previously
	* described, and returns it. It is used by onConverterPopup().
	*/
	registerConverters:function()
	{
	  var conv_symbols=new Array();
	  var conv_functions=new Array();
	  var conv_targets=new Array();
	  
	  var atoms=MCE.conv_table.split(" ");
	  var conv_symbol=true;
	  var register_extended=true;
	  if (MCE.extended_symbols.length) {
	    register_extended=false;
	  }
	  
	  var i;
	  var a_converter_symbol_1=new Array();
	  var a_converter_symbol_2plus=new Array();
	  
	  for(i=0;i<atoms.length;i++) {
	    var atom=atoms[i];
	    if (!atom) {
	       continue;
	    }
	    if (conv_symbol) {
	      if (register_extended && (atom.indexOf('_')>-1)) {
		    MCE.extended_symbols_raw.push(atom.replace(/_/g,' '));
		    MCE.extended_symbols.push(atom.replace(/_/g,'[\\s]+'));
	      }
	      conv_symbols.push(atom);
	      var letter=atom.substring(0,1);
	      if (MCE.util.inArray(letter,a_converter_symbol_1)==-1) {
		    a_converter_symbol_1.push(letter);
		    MCE.symbol_1+=letter;
	      }
	      var j=0;
	      for(j=1;j<atom.length;j++) {
		    letter=atom.substring(j,j+1);
		    if (MCE.util.inArray(letter,a_converter_symbol_2plus)==-1) {
		      a_converter_symbol_2plus.push(letter);
		      MCE.symbol_2plus+=letter;
		    }
	      }
	    } else {
		  if (atom.indexOf("\t")==-1) {
			conv_functions.push(atom);
			conv_targets.push(false);
		  } else {
			var x=atom.split("\t");
			conv_functions.push(x[0]);
			conv_targets.push(x[1]);
		  }
	    }
	    conv_symbol=!conv_symbol;
	  }
	  MCE.symbol_1=this.regexpise(MCE.symbol_1);
	  MCE.symbol_2plus=this.regexpise(MCE.symbol_2plus);
	  
	  return new Array(conv_symbols,conv_functions,conv_targets);
	},

	/**
	* Transforms a string containing raw, individual characters to be matched
	* by regexp to a form which is actually understood by match()
	*/
	regexpise:function(in_s)
	{
	//  alert('in_s: '+in_s);
	  var i=0;
	  var result='';
	  var s=in_s;
	  var re_raw=/[0-9a-zA-Z !"#$%&'*+,.:;<=>?@_`{|}~]/;
	  var re_escape=/[\(\)\-\/\[\\\]\^]/;
	  for(i=0;i<s.length;i++) {
	    var letter=s.substring(i,i+1);
	    var replace=letter;
	    // don't forget about the underscore -- it's never matched here!
	    // standard stuff, goes as is
	    if (!re_raw.exec(letter)) {
	      // stuff that simply needs to be escaped with a backslash
	      if (re_escape.exec(letter)) {
		replace='\\'+replace;
	      } else {
		var hex=replace.charCodeAt(0).toString(16).toUpperCase();
		while(hex.length<4) {
		  hex='0'+hex;
		}
		replace='\\u'+hex;
	      }
	    }
	    result+=replace;
	  }
	//  alert('result: '+result);
	  return result;
	},

	convertPage:function()
	{
	  MCE.iface.setOldPage();
	  window.setCursor('wait');
	  if (MCE.util.regressionTest()) {
	    window.setCursor("auto");
	    return true;
	  }
	  var myDocs=MCE.iface.getAllDocuments();
	  var i;
	  for(i=0;i<myDocs.length;i++) {
	    this.convertDoc(myDocs[i]);
	  }
	  window.setCursor("auto");
	  return true;
	},

	convertDoc:function(aDoc)
	{
	  if (aDoc.getElementsByTagName('BODY').length) {
            this.convertElement(aDoc.getElementsByTagName('BODY')[0],'');
	    return true;
	  }
	  return false;
	},

	restorePage:function()
	{
          var myDocs=MCE.iface.getAllDocuments();
          var i;
          for(i=0;i<myDocs.length;i++) {
            this.restoreDoc(myDocs[i]);
          }
	  return true;
	},

	restoreDoc:function(aDoc)
	{
	  MCE.iface.setNewPage();
	  if (aDoc.getElementsByTagName('BODY').length) {
	    this.restoreElement(aDoc.getElementsByTagName('BODY')[0],'');
            return true;
          }
	  return false;
	},

	restoreElement:function(elem)
	{
	  do {
	    if (elem.converter_originalNode!=undefined) {
	      // Yes, this insanity really is needed, otherwise elem's nextSibling is messed up
	      var replacing=elem;
	      elem=elem.nextSibling;
	      replacing.parentNode.replaceChild(replacing.converter_originalNode,replacing);
	      continue;
	    }
	    if (elem.firstChild!=undefined) {
	      this.restoreElement(elem.firstChild);
	    }
	  } while ((elem!=undefined) && (elem=elem.nextSibling));
	  return true;
	},

	convertElement:function(docu,history)
	{
	  var d=docu;
	  if (d==undefined)
	    return false;
	  var myResult=false;
	  var d_original;
	  do {
	    if (d.nodeType==1) {
	      if (d.nodeName!='SCRIPT' && d.nodeName!='STYLE') {
                d_original=d.cloneNode(true);
	        if (this.convertElement(d.firstChild,history+'/'+d.nodeName)) {
                  // Not bubbling by default, that would cause all parents to get cached!
                  d.converter_originalNode=d_original;
	          // But we do want to bubble if the previous sibling's got it,
	          // or else its backup node will point to this node's original,
	          // which doesn't contain the backup. Hairy.
	          if (d.previousSibling!=undefined && d.previousSibling.converter_originalNode!=undefined) {
	            myResult=true;
	          }
                }
	      }
	    } else if (d.nodeType!=8) { // not converting HTML comments
	      var d_content=d.nodeValue;
	      var skip_sibling=false;
	      if (d.nextSibling!=undefined && d.nextSibling.nodeName=='SUP') {
	        var sup_index=d_content.length;
	        d_content=d_content+'^'+d.nextSibling.firstChild.nodeValue;
	        skip_sibling=true;
	      }
	      var re=/[0-9]/;
	      var result=re.exec(d.nodeValue);
	      if (!result) {
	        d=d.nextSibling;
	        if (skip_sibling) {
	        skip_sibling=false;
	          if (d!=undefined)
	            d=d.nextSibling;
	        }
	        continue;
	      }
	      //alert('Converting this: '+d.nodeValue);
	      var new_content=this.convertText(d_content);
	      //d.nodeValue=this.convertText(d_content);
	      if (new_content!=d_content) {
	        myResult=true;
	        var new_index=sup_index+new_content.length-d_content.length;
	        if (new_content.substring(new_index)!=d_content.substring(sup_index)) {
	          d.nodeValue=new_content;
	          if (skip_sibling) {
	            d.parentNode.removeChild(d.nextSibling);
	          }
	        }
	      }
	    };
	    d=d.nextSibling;
	    if (skip_sibling) {
	      skip_sibling=false;
	      if (d!=undefined)
	        d=d.nextSibling;
	    }
	    // debug: alert('d.nodeType='+d.nodeType+'; d.nodeName='+d.nodeName+'; d.nodeValue='+d.nodeValue);
	  } while (d!=undefined);
	  return myResult;
	},

	convertText:function(txt)
	{
	  var rawText=txt; // this is the original text which gets chopped
	  //debug: alert('text before: |'+rawText+'|');
	  var re = /\r/g;
	  rawText=rawText.replace(re,' ');
	  re = /\n/g;
	  rawText=rawText.replace(re,' ');
	  //debug: alert('text after: |'+rawText+'|');
	  var convertedText=''; // this is the converted text which grows
	  var wordData=false; // temporary var where we store the result of cGSW()
	  var convertedTmp='';

	  wordData=this.getSelectedWord(rawText);
	  if (!wordData) {
	    // If there's nothing to convert, return the original result
	    return txt;
	  }
	  while (wordData) {
	    var cr=this.doConversion(wordData.match,wordData);
	    if (MCE.prefs.getPref('pref_replace_original') && cr && !cr.inhibit) {
	      convertedText=convertedText+rawText.substring(0,wordData.index);
	    } else {
	      convertedText=convertedText+rawText.substring(0,wordData.index+wordData.match.length);
	    }
	    rawText=rawText.substring(wordData.index+wordData.match.length,rawText.length);
	    if (cr && !cr.inhibit) {
	      if (cr.only_show_converted || MCE.prefs.getPref('pref_only_show_converted')) {
		    convertedTmp=cr.outCustom;
	      } else {
		    convertedTmp=cr.inValue + " " + cr.inUnit + " = " + cr.outCustom;
	      }
	      if (MCE.prefs.getPref('pref_replace_original')) {
		convertedText+=convertedTmp;
	      } else {
		convertedText+=" ("+convertedTmp+")";
	      }
	    }
	    wordData=this.getSelectedWord(rawText)
	  }
	  convertedText=convertedText+rawText;
	  return convertedText;
	},

	computeFraction:function(text)
	{
		var re = /([0-9]+)\/([0-9]+)/;
		var result=re.exec(text);
		if (!result) {
			// We assume we're receiving a number, at least
			return new Number(text);
		}
		var numr=new Number(result[1]);
		var numt=new Number(result[2]);
		if (!numr || !numt) {
			return 0;
		}
		return numr/numt;
		
	},

	getSelectedWord:function(word)
	{
	  var results=[];
	  var tmp;

	  // Let's start by checking for the awkward Imperial notations.
	  
	  // First checking for egineering format (1'2 1/32" or 1'2-1/32").
	  // If found, we convert the whole thing to inches and return as if we
	  // encountered the corresponding string in inches using decimal notation
	  // instead of fractions.
	  var re = /(([0-9]*)\')?([0-9]+)[\s\-]([0-9]+\/[0-9]+)[\s\-]?(\"|inches|inch|in)/i;
	  var result = re.exec(word);
	  if (result) {
	    if (!result[1])
	      result[1]=0;
	    tmp={
	      parser: 'awkward Imperial'
	    };
	    tmp.value=new Number(0);
	    if (result[2]!=undefined) {
	      tmp.value+=new Number(12*result[2]);
	    }
	    tmp.value+=new Number(result[3]);
	    tmp.value+=this.computeFraction(result[4]);
	    tmp.unit="in";
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    results.push(tmp);
	  }
	  
	  var re = /([0-9]*)\'([0-9]+)[\s\-]([0-9\/]+)/;
	  var result = re.exec(word);
	  if (result) {
	    tmp={
	      parser: 'awkward Imperial II'
	    };
	    tmp.value=12*result[1]+1*result[2]+this.computeFraction(result[3]);
	    tmp.unit="in";
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    results.push(tmp);
	  }
	  
	  // And finally checking for informal format (5'10)
	  // We first try finding the full format (x'y"), and if that doesn't work,
	  // we look for the simplified format (x'y) -- unfortunately we can't append
	  // ["]? at the end of the first regexp, because it would never match the
	  // final double quote, even if it's there, since it's optional and there's
	  // nothing afterwards to force the regexp algorithm to use it.
	  var re = /([0-9]+)\'[ ]*([0-9]{1,2})\"/;
	  var result = re.exec(word);
	  if (result) {
	    tmp={
	      parser: 'informal Imperial'
	    };
	    tmp.value=12*result[1]+1*result[2];
	    tmp.unit="in";
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    results.push(tmp);
	  } else {
	    var re = /([0-9]+)\'[ ]*([0-9]{1,2})/;
	    var result = re.exec(word);
	    if (result) {
	      tmp={
	        parser: 'informal Imperial II'
	      };
	      tmp.value=12*result[1]+1*result[2];
	      tmp.unit="in";
	      tmp.match=result[0];
	      tmp.index=word.indexOf(result[0]);
	      results.push(tmp);
	    }
	  }

	  // Let's check for extended symbols
	  // Double backslashes below because we need to escape the backslash itself to
	  // get it across the string format. One hour of cursing; idiot, I should have known this.
	  // Changed
	  // ([\\-]?[0-9.][0-9.,\\/]*)[\\s]+
	  // to
	  // ([\\-]?[0-9.][0-9.,\\/]*)[\\s]*
	  // in order to accept "10&deg; F"
	  var re = new RegExp("([\\-]?[0-9.][0-9.,\\/]*)[\\s]*("+MCE.extended_symbols.join('|').replace(/\./g,"\\.")+")","i");
	  var result = re.exec(word);
	  if ((result) && (result[1]) && (result[2])) {
	    var pos=MCE.util.inArray(result[2].toLowerCase().replace(/[\s]+/g,' '),MCE.extended_symbols_raw);
	    if (pos>-1)
	    {
	      //debug: alert('extended match: '+result[1]+'--'+result[2]);
	      tmp={
	        parser: 'extended symbols'
	      };
	      tmp.value=result[1];
	      tmp.unit=MCE.extended_symbols_raw[pos].replace(/ /g,'_');
	      tmp.match=result[0];
	      tmp.index=word.indexOf(result[0]);
	      results.push(tmp);
	    }
	  }
	  
	  // 12h/24h
	  //var re = /([0-9]{1,2})(:([0-9]{2}))?(:([0-9]{2}))?[\s]*(am|a\.m\.|pm|p\.m\.)?[\s]*([+\-0-9A-Z]{3,5})?/i;
	  /*
	    This needs to be broken down in two cases ("hh:mm" plus optional stuff, and
	    "hh AM/PM" plus optional stuff); making both ":mm" and "AM/PM" optional
	    results in messy matches which break the processing.
	    ----------
	    Update: the first case above needs to be broken down in two cases as well:
	    "<hh:mm> [am/pm] <TZ>" and "<hh:mm> [am|pm]" in order to properly parse
	    "abc 9:00 AM cba -- abc 9:00 AM cbaza" -- and in the first branch, the <TZ>
	    needs to actually be verified.
	    ----------
	    Update: in the TZ area, the [\s]* is included explicitly because it needs
	    to live in the TZ atom; otherwise "local 9:00 AM in Austria" is improperly
	    spaced.
	    ----------
	    Update: if/else doesn't work well when multiple time formats are shown
	    one after another -- "10 a.m. * 10 p.m. * 10 A.M. * 10 P.M. * 13:33:33 UTC"
	    ----------
	    Update: the military format (0800 GMT) is used more frequently than I
	    had assumed; made the colon optional in all formats in order to support it.
	    ----------
	    Update: the conditions for tmp used to be as follows:
	    if (result && result[1] && (!result[3] || (result[4] && MCE.tzUtil.get_tz_offset(result[4])!=undefined) || (result[5] && MCE.tzUtil.get_tz_offset(result[5])!=undefined))) {

	    I have removed the "!result[3]" condition because it didn't seem to make
	    sense -- result[3] is a blanket generalization and I can't currently
	    understand why it was there in the first place...
	  */
	  var re = /([0-9]{1,2}):?([0-9]{2})(:([0-9]{2}))?[\s]*(am|a\.m\.|pm|p\.m\.)?([\s]*([+\-][0-9]{4})|[\s]*([a-z]{3,4}))?/i;
	  var result = re.exec(word);
	  if (
	    result &&
	    MCE.tzUtil.validate_hour(result[1]) &&
	    MCE.tzUtil.validate_minute(result[2]) &&
	    (
	      (result[5] && !result[6]) ||
	      (
	        result[7] &&
	        MCE.tzUtil.get_tz_offset(result[7])!=undefined
	      ) ||
	      (
	        result[8] &&
	        MCE.tzUtil.get_tz_offset(result[8])!=undefined
	      )
	    )
	  ) {
	    tmp={
	      parser: 'TZ I'
	    };
	    tmp.value=result[0];
	    tmp.unit='internal_time'
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    results.push(tmp);
	  }

	  // UTC+N, UTC-N, GMT+N, GMT-N
	  var re = /([0-9]{1,2}):([0-9]{2})(:([0-9]{2}))?[\s]*[\(]?(UTC|GMT)([+\-])([0-9]{1,2})[\)]?/;
	  var result = re.exec(word);
	  if (result) {
	    tmp={
	      parser: 'TZ UTC',
	      value: result[0],
	      unit: 'internal_time',
	      match: result[0],
	      index: word.indexOf(result[0]),
	      orig_tz: new Number(result[6]+result[7])
	    };
            results.push(tmp);
	  }


	  // Added version with mandatory colon, because there are some formats
	  // which were "tricking" the optional regexp
	  var re = /([0-9]{1,2}):([0-9]{2})(:([0-9]{2}))?[\s]*(am|a\.m\.|pm|p\.m\.)?([\s]*([+\-][0-9]{4})|[\s]*([a-z]{3,4}))?/i;
	  var result = re.exec(word);
	  if (
	    result &&
	    MCE.tzUtil.validate_hour(result[1]) &&
	    MCE.tzUtil.validate_minute(result[2]) &&
	   (
	     !result[6] ||
	     (
	       result[7] &&
	       MCE.tzUtil.get_tz_offset(result[7])!=undefined) ||
	       (
	         result[8] &&
	         MCE.tzUtil.get_tz_offset(result[8])!=undefined
	       )
	     )
	   ) {
	    tmp={
	      parser: 'TZ II'
	    };
	    tmp.value=result[0];
	    tmp.unit='internal_time'
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    results.push(tmp);
	  }

	  // There used to be a question mark after the am/pm test, but I removed it
	  // because that was matching (and converting) "2009" and such.
	  var re = /([0-9]{1,2}):?([0-9]{2})(:([0-9]{2}))?[\s]*(am|a\.m\.|pm|p\.m\.)/i;
	  var result = re.exec(word);
	  if (
	    result &&
	    MCE.tzUtil.validate_hour(result[1]) &&
	    MCE.tzUtil.validate_minute(result[2])
	  ) {
	    tmp={
	      parser: 'TZ III'
	    };
	    tmp.value=result[0];
	    tmp.unit='internal_time'
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    results.push(tmp);
	  }

	  var re = /([0-9]{1,2})[\s]*(am|a\.m\.|pm|p\.m\.)([\s]*([+\-][0-9]{4})|[\s]*([A-Z]{3,4}))?/i;
	  var result = re.exec(word);
	  if (
	    result &&
	    MCE.tzUtil.validate_hour(result[1]) &&
	    (
	      (
	        !result[4] &&
	        !result[5]
	      ) ||
	      (
	        result[4] &&
	        MCE.tzUtil.get_tz_offset(result[4])!=undefined
	      ) ||
	      (
	        result[5] &&
	        MCE.tzUtil.get_tz_offset(result[5])!=undefined
	      )
	    )
	  ) {
	    if (!result[3]) {
	      result[3]='';
	    } else {
	      result[3]=' '+result[3];
	    }
	    tmp={
	      parser: 'TZ IV'
	    };
	    tmp.value=result[1]+':00 '+result[2]+result[3];
	    tmp.unit='internal_time'
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    results.push(tmp);
	  }

	  // Ok, didn't find any of the wicked ones, let's check for
	  // some reasonable measurements AND fractions.
	  // --- NOTES ---
	  // The last bracket used to include "\-", but I removed it as to allow
	  // converting all of those Imperial idiots' "50-foot monster" and so on.
	  // Hope I didn't mess up, I can't remember whether there was a real reason
	  // for that in here, or whether it was simply a precaution.
	  // -------------
	  // Explicityly avoiding the "-" units because it used to match "2008-10-12" as
	  // "2008" with units "-".
	  // -------------
	  // Also, I replaced the second atom,
	  // ([^ .,!?;+\(\)0-9]+)
	  // with
	  // ([^ .,!?;+\(\)0-9][^ .,!?;+\(\)0-9'"]*)
	  // as to allow stuff in quotes -- e.g. "25 kg" -- without messing up the
	  // support for inches (") and feet (').
	  // -------------
	  // Disallowing numbers in the first position for units, because it was
	  // breaking dates
	  // ("Thu, 21 Dec 2000 16:59 -0030" --> it was catching "2000 1")
	  // -------------
	  // Added a "rough" regexp test before the actual test in order to rule out
	  // long strings made of numbers, which made the regexp freeze for way
	  // too long.
	  // -------------
	  // Changed exclusive tests ([^foo]+) with inclusive tests ([bar]+) in order
	  // to avoid various types of mishaps (I had to constantly adjust the
	  // restrictions, and I always failed making it work for all situations).
	  // The CVS version which still contains (imperfect) exclusive tests is 1.42.
	  // -------------
	  // The reason why I allow ([\\d]+[.]?)+ instead of ([\\d])+[.]? is that
	  // I have to account for "6.322.867 feet". Of course, that shouldn't
	  // interfere with 'disabling "Converter 0.6.2" stops FF from freezing',
	  // which used to happen (it used to converte "062 in" instead of "6.2 in").
	  // - END NOTES -

	  // Not using the "proper" stuff here, we just need a rough test (never tested
	  // this version, but it's most probably too strict to be useful, because we
	  // include restrictions based on units, but be DON'T allow stuff
	  // related to other considerations ("thousands", number formatting, etc)
	  //var re = new RegExp("([\\-]?)([\\d,.]+)[\\s]*(["+MCE.symbol_1+"]["+MCE.symbol_2plus+"]+)");
	  // ok, now that we have proper, identical rules for comma- and
	  // period-fractions, we need better safeguards too.
	  // var re = /([\-]?)([\d,.]+)[ ]*([^ .,!?;+\(\)0-9\s]+)/[ignore this bracket]

	  var re = /([\-]?)([\d]+\.|[\d]+|\.|[\d]+,)([\s]*|-)([\d]*)([^ .,!?;+\(\)0-9\s]+)/
	  if (re.exec(word)) {
	    var re = new RegExp("([\\-]?)(((([\\d]+[.]?)+(,[0-9]+)?)|([\\d]*\\.[\\d]+))([/][0-9]+)?)[\\s]*(thousand|million)?[\\s\\-]*(["+MCE.symbol_1+"]["+MCE.symbol_2plus+"]*)","i");
	    var result = re.exec(word);
	    //debug: alert('Matching this: '+word);
	    // Must have the first (result), or the whole thing blows for null results
	    if ((result) && (result[2]) && (result[10]) && (result[10].substring(result[10].length-1)!='-')) {
	      //debug: alert('Match -- ['+result[1]+'] -- ['+result[2]+']');
	      var tmpTest=result[2];
	      var tmp_idx=result[2].indexOf("/");
	      if (tmp_idx!=-1 && tmp_idx!=result[2].length-1) {
		var resultMembers=result[2].split("/");
		for(i=0;i<resultMembers.length;i++) {
		  resultMembers[i]=MCE.util.determine_separators(resultMembers[i]);
		}
		result[2]=this.computeFraction(resultMembers.join("/"));
	      }
	      var multi=1;
	      if (result[9]) {
		if (result[9]=='million') {
		  multi=1000000;
		} else {
		  multi=1000;
		}
	      }
	      tmp={
	        parser: 'regular comma'
	      };
	      tmp.value=result[1]+result[2];
	      tmp.unit=MCE.util.simplify(result[10].toLowerCase());
	      tmp.match=MCE.util.simplify(result[0]);
	      tmp.index=word.indexOf(result[0]);
	      tmp.multiplier=multi;
	      results.push(tmp);
	    }

	    var re = new RegExp("([\\-]?)((([\\d]+[\\,]?)+(\\.[0-9]+)?)([/][0-9]+)?)[\\s]*(thousand|million)?[\\s]*(["+MCE.symbol_1+"]["+MCE.symbol_2plus+"]*)","i");
	    var result = re.exec(word);
	    //debug: alert('Matching this: '+word);
	    // Must have the first (result), or the whole thing blows for null results
	    if ((result) && (result[2]) && (result[8]) && (result[8].substring(result[8].length-1)!='-')) {
	      //debug: alert('Match -- ['+result[1]+'] -- ['+result[2]+']');
	      var tmpTest=result[2];
	      var tmp_idx=result[2].indexOf("/");
	      if (tmp_idx!=-1 && tmp_idx!=result[2].length-1) {
		var resultMembers=result[2].split("/");
		for(i=0;i<resultMembers.length;i++) {
		  resultMembers[i]=MCE.util.determine_separators(resultMembers[i]);
		}
		result[2]=this.computeFraction(resultMembers.join("/"));
	      }
	      var multi=1;
	      if (result[7]) {
		if (result[7]=='million') {
		  multi=1000000;
		} else {
		  multi=1000;
		}
	      }
	      tmp={
	        parser: 'regular period'
	      };
	      tmp.value=result[1]+result[2];
	      tmp.unit=MCE.util.simplify(result[8].toLowerCase());
	      tmp.match=MCE.util.simplify(result[0]);
	      tmp.index=word.indexOf(result[0]);
	      tmp.multiplier=multi;
	      results.push(tmp);
	    }
	  }

	  // Explicitly looking for 10 foot 5, 10-foot-6, "10 feet 5 inches", etc
	  // Note: I removed support for "10 feet, 5 inches" because it was messing
	  // up legitimate lists like "10 feet, 12 feet" -- if this is really needed
	  // it must be separated into a new test.
	  // Be advised that this is quite finely tuned, don't mess with it lightly.
	  re = /([0-9]+)[\s\-]*(foot|feet|ft|'|`)[\s\-]*([0-9]*(\.[0-9]+)?)[\s\-]*?(([0-9]+)\/([0-9]+)[\s\-]*)?([\s]*(inches|inch|in))?/i;
	  result = re.exec(word);
	  if (result) {
	    var fract=0;
	    if (result[5] && (result[7]>0)) {
	      fract=result[6]/result[7];
	    }
	    tmp={
	      parser: 'explicit 10 foot 5'
	    };
	    if (result[3] || fract) {
	      tmp.value=12*result[1]+1*result[3]+fract;
	      tmp.unit='in';
	    } else {
	      tmp.value=result[1];
	      tmp.unit='ft';
	    }
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    results.push(tmp);
	  }

	  // Explicitly looking for 10 foot 1/5, 10-foot-1/6, etc
	  // This is an exact replica of the above, but without the integer part for inches
	  re = /([0-9]+)[\s\-]*(foot|feet|ft|'|`)[\s\-]*(([0-9]+)\/([0-9]+)[\s\-]*)([\s]*(inches|inch|in))?/i;
	  result = re.exec(word);
	  if (result && result[5]>0) {
	    tmp={
	      parser: 'explicit 10 foot 1/5'
	    };
	    tmp.value=12*result[1]+result[4]/result[5];
	    tmp.unit='in';
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    results.push(tmp);
	  }

	  // Explicitly looking for 11 stone 4
	  re = /([0-9]+)[\s]+stone(s)?[\s]+([0-9]+)/;
	  result = re.exec(word);
	  if (result) {
	    tmp={
	      parser: 'explicit 11 stone 4'
	    };
	    tmp.value=Number(result[1])+Number(result[3])/14;
	    tmp.unit='stones';
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    results.push(tmp);
	  }

	  // Currency conversions
	  if (MCE.prefs.getPref('pref_currency_enabled')) {
	    results=results.concat(this.getCurrencyWord(word));
	  }

	  return this.bestAmong(results);
	},

	getCurrencyWord:function(word)
	{
	  var results=[];
	  var tmp;
	  var re;
	  var result;
	  var i;
	  var number_regexp_next=6;
          var number_regexp=
	    "("+
	      "([\\d]+\\,)+\\.[\\d]+|"+ // 123,123,123.123456
	      "([\\d]+\\.)+\\,[\\d]+|"+ // 123.123.123,123456
	      "([\\d]+\\,)+[\\d]+|"+ // 123,123,123
	      "([\\d]+\\.)+[\\d]+|"+ // 123.123.123
	      "\\.[\\d]+|"+ // .123456
	      "[\\d]+"+ // 123456
	    ")";

	  // First, a rough test to exclude long strings of digits
	  if (/[\d.,]+[\s]*[a-z]{3}/i.exec(word)) {
	    re = new RegExp(number_regexp+"[\\s]*([a-z]{3})([a-z]?)","i");
	    result = re.exec(word);
	    if (
	      result &&
	      !result[number_regexp_next+1] && /* We don't want to convert "3 cops" ("3 COP") */
	      MCE.util.inArray(result[number_regexp_next].toUpperCase(),MCE.currency.currencies)>-1
	    ) {
	      tmp={
	        parser: 'currency, regular, right'
	      };
	      tmp.value=result[1];
	      tmp.unit='internal_currency';
	      tmp.match=result[0];
	      tmp.index=word.indexOf(result[0]);
	      tmp.currency=result[number_regexp_next].toUpperCase();
	      results.push(tmp);
	    }
	  }

	  re = new RegExp("([a-z]?)([a-z]{3})[\\s]*"+number_regexp,"i");
	  result = re.exec(word);
	  if (
	    result &&
	    !result[1] && /* We don't want to convert "fleur 3" ("EUR 3") */
	    MCE.util.inArray(result[2].toUpperCase(),MCE.currency.currencies)>-1
	  ) {
	    tmp={
	      parser: 'currency, regular, left'
	    };
	    tmp.value=result[3];
	    tmp.unit='internal_currency';
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    tmp.currency=result[2].toUpperCase();
	    results.push(tmp);
	  }

	  // First, rough test to rule out long strings of digits
	  if (/[\d.,]+[\s]*[^\s\d.,]/.exec(word)) {
	    re = new RegExp(number_regexp+"[\\s]*(["+MCE.currency.unambiguous_regexp+"])");
	    result=re.exec(word);
	    if (result) {
	      tmp={
	        parser: 'currency, unambiguous, right'
	      };
	      tmp.value=result[1];
	      tmp.unit='internal_currency';
	      tmp.match=result[0];
	      tmp.index=word.indexOf(result[0]);
	      for(i=0;i<MCE.currency.unambiguous_currency_symbols.length;i++) {
	        if(result[number_regexp_next]==MCE.currency.unambiguous_currency_symbols[i].symbol) {
	          tmp.currency=MCE.currency.unambiguous_currency_symbols[i].ISO;
	          break;
	        }
	      }
	      results.push(tmp);
	    }
	  }

	  re = new RegExp("(["+MCE.currency.unambiguous_regexp+"])[\\s]*"+number_regexp);
	  result=re.exec(word);
	  if (result) {
	    tmp={
	      parser: 'currency, unambiguous, left'
	    };
	    tmp.value=result[2];
	    tmp.unit='internal_currency';
	    tmp.match=result[0];
	    tmp.index=word.indexOf(result[0]);
	    for(i=0;i<MCE.currency.unambiguous_currency_symbols.length;i++) {
	      if(result[1]==MCE.currency.unambiguous_currency_symbols[i].symbol) {
	        tmp.currency=MCE.currency.unambiguous_currency_symbols[i].ISO;
	        break;
	      }
	    }
	    results.push(tmp);
	  }

	  return results;
	},

	/*
	* This method compares any number of atoms.
	*/
	bestAmong:function(results)
	{
	  if (!results.length) {
	    return false;
	  }
	  var br=results[0];
	  // Trimming the fat at the end
	  br.match=br.match.replace(/[\s]+$/,'');

	  for(var i=1;i<results.length;i++) {
	    results[i].match=results[i].match.replace(/[\s]+$/,'');
	    br=this.bestBetween(br,results[i]);
	  }
	  return br;
	},

	/*
	* This method compares two atoms and returns the best fit.
	*
	* By "best fit" I understand applying the following criteria, in this
	* order (i.e. if all things are equal, go on; otherwise, make a decision):
	* 1. The other is no match or it's an all-numeric match (i.e. the matching
	*    string is either empty or only made of digits and spaces);
	* 2. The other shows up later in the original string;
	* 3. The other is shorter;
	* 4. The other's VALUE is smaller -- this is here in order to prevent
	*    Aresult3 (generic) from hijaking Aresult2 (e.g. "3'3"). Given that
	*    this is practically the last meaningful test -- all hh:mm and so on
	*    should have already been sorted out by now, and returing the first one
	*    randomly (as the next decision would) can't be worse than comparing
	*    their magnitudes.
	* 5. Everything's equal so far. But if one of them is currency then we give
	*    that one priority, because we make strict tests for currency validity
	*    on parsing, whereas other units are not so strictly controlled.
	* 6. Return the first one. Well, a decision needs to be made, after all.
	*    Thankfully this happens practically never, and when it does the results are
	*    typically identical anyway (i.e. the same conversion is triggered).
	*    TODO: I could study this in more depth though, maybe by phoning home when
	*    this happens and the units are not the same (with the user's permission,
	*    of course).
	*/
	bestBetween:function(res1,res2)
	{
	  var re=/^[0-9 ]+$/;
	  // (1)
	  if (!res1.match || re.exec(res1.match)) {
	    return res2;
	  }
	  if (!res2.match || re.exec(res2.match)) {
	    return res1;
	  }
	  // (2)
	  if (res1.index<res2.index) {
	    return res1;
	  }
	  if (res1.index>res2.index) {
	    return res2;
	  }
	  // (3)
	  if (res2.match.length>res1.match.length) {
	    return res2;
	  }
	  if (res2.match.length<res1.match.length) {
	    return res1;
	  }
	  // (4)
	  if (res2.value>res1.value) {
	    return res2;
	  }
	  // (5)
	  if (res1.currency!==undefined) {
	    return res1;
	  }
	  if (res2.currency!==undefined) {
	    return res2;
	  }
	  // (6)
	  return res1;
	},

	init:function()
	{
	  MCE.extended_symbols=new Array();
	  MCE.extended_symbols_raw=new Array();
	  MCE.symbol_1='';
	  MCE.symbol_2plus='';
	  MCE.conversions = MCE.core.registerConverters();
	}
}

MCE.core.init();
}
